/*
* =====================================================================
*  Filename: TradingSystemReqDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxdtctl.h"
#include "NumEdit.h"
#include "Trader.h"

// CAutoTraderReqDlg dialog
class CAutoTraderReqDlg : public CDialog
{

public:

   /// \brief CAutoTraderReqDlg constructor.
   /// \param spRequest - ICQGTradingSystemRequest object
   /// \param lineTime - line time from CQGCEL
   /// \param pParent - parent window
   CAutoTraderReqDlg(
               const ICQGTradingSystemRequestPtr& spRequest,
               const ICQGAccountsPtr& spAccounts,
               CWnd* pParent = NULL);

   /// \enum Dialog Data.
   enum { IDD = IDD_AUTO_TRADER_REQ_DIALOG };

private:

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
               CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

private:

   /// \brief Sets request parameters and closes the form.
   afx_msg void OnBnClickedOk();

   /// \brief Used to increment or decrement integer range start.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRangeStart(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement integer range end.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRangeEnd(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement intra day period.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnIntraday(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement days before expiration.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnDaysBeforeExp(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement sessions filter.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnSessionsFilter(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Changes historical type and enables/disables relative controls.
   afx_msg void OnCbnSelChangeCmbHistorical();

   /// \brief Checks/unchecks custom session and unhides/hides relative controls.
   afx_msg void OnBnClickedChkCustomSession();

   /// \brief Changes continuation type and enables/disables relative controls.
   afx_msg void OnCbnSelChangeCmbContinuation();

   /// \brief Changes parameter's value to the given in the list.
   afx_msg void OnBnClickedBtnChangeValue();
   
   /// \brief Changes selected parameter.
   afx_msg void OnLvnItemchangedLsvParameters(
               NMHDR *pNMHDR, 
               LRESULT *pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief Returns user specified auto trader name.
   /// \return Auto trader name
   CString GetAutoTraderName() const;

   /// \brief Returns user selected account ID.
   /// \return Gateway account ID
   long GetAccountID() const;

private:

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes parameters list.
   void InitializeParametersList();

   /// \brief Sets all parameters default values taken from CQGTradingSystemRequest.
   void SetDefaultParameters();

   /// \brief Adds parameters into the list.
   void AddParameters();

   /// \brief Initializes accounts combo box.
   void InitializeAccountsCombo();

   /// \brief Initializes historical periods combo box.
   void InitializeHistoricalCombo();

   /// \brief Initializes continuation types combo box.
   void InitializeContinuationCombo();
   
   /// \brief Checks fields in the form according to the specified session flags bit mask.
   /// \param sessionFlag - sessions flag bit mask
   void SetSessionFlags(
               eSessionFlag sessionFlag);

   /// \brief Returns session flag bit mask according to the checked fields.
   /// \return Session flag as eSessionFlag object.
   eSessionFlag GetSessionFlags();

private:

   /// \enum eParametersColumns. Represents parameters columns in the list.
   enum eParametersColumns;

private:

   /// \brief CQGTradingSystemRequest object.
   ICQGTradingSystemRequestPtr m_spRequest;

   /// \brief CQGCQGAccounts object.
   ICQGAccountsPtr m_spAccounts;

   /// \brief Auto trader name.
   CString m_TraderName;

   /// \brief Account ID.
   long m_AccountID;
   
private:

   /// \brief Request's baseExpression.
   CString m_BaseExpression;

   /// \brief Auto trader name Edit control.
   CEdit m_edTraderName;

   /// \brief Accounts combo box control
   /// \brief Contains account's GW names, as well as the GWID as item data.
   CComboBox m_cmbAccounts;

   /// \brief Parameters' ListView control.
   CListCtrl m_lvParameters;

   /// \brief Parameter's new value EditBox control.
   CNumEdit m_edParameterValue;

   /// \brief Change parameter's value Button control.
   CButton m_btnChangeValue;

   /// \brief Request's historical period ComboBox control.
   CComboBox m_cmbHistorical;

   /// \brief Request's intra day period Edit control.
   CNumEdit m_edIntraday;

   /// \brief Request's intraday period SpinButton control.
   CSpinButtonCtrl m_spnIntraday;

   /// \brief Request's continuation type ComboBox control.
   CComboBox m_cmbContinuation;

   /// \brief Request's equalize closes type CheckBox control.
   CButton m_chkEqualizeCloses;

   /// \brief Request's equalize closes.
   BOOL m_EqualizeCloses;

   /// \brief Request's days before expiration EditBox control.
   CNumEdit m_edDaysBeforeExp;

   /// \brief Request's days before expiration SpinButton control.
   CSpinButtonCtrl m_spnDaysBeforeExp;

   /// \brief Request's daily from intraday.
   BOOL m_DailyFromIntraday;

   /// \brief Request's sessions filter EditBox control.
   CNumEdit m_edSessionsFilter;

   /// \brief Request's sessions filter SpinButton control.
   CSpinButtonCtrl m_spnSessionsFilter;

   /// \brief Request's custom sessions filter.
   CString m_CustomSessionsFilter;

   /// \brief Request's custom sessions filter EditBox control.
   CEdit m_edCustomSessionsFilter;

   /// \brief Request's custom session.
   BOOL m_CustomSession;
};

enum CAutoTraderReqDlg::eParametersColumns
{
   pcName,
   pcComment,
   pcType,
   pcDefaultValue,
   pcValue
};