// Trader.cpp : implementation file
//

#include "StdAfx.h"
#include "Helper.h"
#include "Trader.h"

namespace
{
   inline bool IsValidPosition(long position)
   {
      return (position != LONG_MIN);
   }
}

Trader::Trader(
         const ICQGCELPtr& spCQGCEL,
         const CString& autoTraderName,
         long accountID,
         const ICQGTradingSystemPtr& spTradingSystem)
   :  m_spCQGCEL(spCQGCEL),
      m_Name(autoTraderName),
      m_AccountID(accountID),
      m_spTradingSystem(spTradingSystem),
      m_Symbol(ATL::CW2T(spTradingSystem->GetRequest()->GetBaseExpression())),
      m_OpenPositions(LONG_MIN),
      m_IsActivated(false)
{
   m_spCQGCEL->NewInstrument(_bstr_t(m_Symbol));
}

Trader::~Trader(void)
{
}

void Trader::Activate(bool status)
{
   m_IsActivated = status;
   m_ActivationTime = COleDateTime::GetCurrentTime();
}

CString Trader::GetActivationTime()
{
   return (m_IsActivated ? Common::DateTimeToString(m_ActivationTime) : CHelper::N_A);
}

CString Trader::GetAccountName()
{
   ICQGAccountPtr acc = m_spCQGCEL->GetAccounts()->GetItem(m_AccountID);

   CString accName;
   if (acc)
   {
      accName = ATL::CW2T(acc->GetGWAccountName());
   } 
   else
   {
      accName = _T("");
   }

   return accName;
}

ICQGAccountPtr Trader::GetAccount()
{
   ICQGAccountPtr acc = m_spCQGCEL->GetAccounts()->GetItem(m_AccountID);
   return acc;
}

ICQGOrderPtr Trader::GetLastOrder()
{
   return m_LastOrder;
}


bool Trader::GetReadyStatus()
{
   const bool tradingSystemResolved = (m_spTradingSystem->GetStatus() == rsSuccess);
   const bool symbolResolved = ((m_InstrumentName != CHelper::N_A) && !m_InstrumentName.IsEmpty());
   const bool accountExist = !GetAccountName().IsEmpty();
   const bool validPosition = IsValidPosition(m_OpenPositions);

   return (tradingSystemResolved && symbolResolved && accountExist && validPosition);
}

CString Trader::GetOpenPositionStr()
{
   CString openPositions = CHelper::N_A;
   if (IsValidPosition(m_OpenPositions))
   {
      openPositions.Format(_T("%d"), m_OpenPositions);
   }

   return openPositions;
}

CString Trader::GetPreferences()
{
   CString prefStr;

   ICQGTradingSystemRequestPtr spRequest = m_spTradingSystem->GetRequest();

   eHistoricalPeriod historicalPeriod = spRequest->GetHistoricalPeriod();
   if (hpUndefined != historicalPeriod)
   {
      prefStr.Format(_T("%s; "),
         static_cast<LPCTSTR>(CHelper::HistoricalPeriodToString(historicalPeriod)));
   }
   else
   {
      prefStr.Format(_T("%s Min; "),
         static_cast<LPCTSTR>(Common::VariantToString(spRequest->GetIntradayPeriod())));
   }
   prefStr.AppendFormat(_T("CNT: %s; "),
      static_cast<LPCTSTR>(CHelper::ContinuationTypeToShortString(spRequest->GetContinuation())));

   prefStr.AppendFormat(_T("DBE: %s; "),
      static_cast<LPCTSTR>(Common::VariantToString(spRequest->GetDaysBeforeExpiration())));

   prefStr.AppendFormat(_T("ECL: %s; "),
      static_cast<LPCTSTR>(Common::VariantToString(spRequest->GetEqualizeCloses())));

   prefStr.AppendFormat(_T("SFL: %s"),
      static_cast<LPCTSTR>(Common::VariantToString(spRequest->GetSessionsFilter())));


   return prefStr;
}

CString Trader::GetParameters()
{
   CString paramsStr = _T("");

   ICQGTradingSystemRequestPtr spRequest = m_spTradingSystem->GetRequest();
   ICQGParameterDefinitionsPtr spParams = spRequest->GetDefinition()->GetParameterDefinitions();

   spRequest->GetDefinition();

   for (int i = 0; i < spParams->GetCount(); ++i)
   {
      CString paramName = spParams->GetItem(i)->GetName();
      CString paramValue = Common::VariantToString(
                  spRequest->GetParameter(ATL::CComVariant(paramName)), CHelper::PARAM_FRAC_PART_LENGTH);
      
      paramsStr.AppendFormat(_T("%s: %s; "),
         static_cast<LPCTSTR>(paramName), static_cast<LPCTSTR>(paramValue));
   }

   return paramsStr;
}

void Trader::SetInstrument(const ICQGInstrumentPtr& val)
{
   m_Instrument = val;
   m_InstrumentName = ATL::CW2T(m_Instrument->GetFullName());
}

bool Trader::TradingSystemChanged()
{
   ICQGTradingSystemStatisticsCollectionPtr statistics = m_spTradingSystem->GetStatistics();

   const long count = statistics->GetCount();

   if (0 == count)
   {
      return false;
   }
   
   const double openPosition = statistics->GetItem(count - 1)->GetOpenPosition();
   if (VARIANT_FALSE != m_spCQGCEL->IsValid(openPosition))
   {      
      const long delta = (static_cast<long>(openPosition) - m_OpenPositions);
      m_OpenPositions = static_cast<long>(openPosition);
      if (m_IsActivated && 0 != delta)
      {
         PlaceOrder(delta);
         return true;
      }
   }

   return false;
}

void Trader::PlaceOrder(
         long quantity)
{
   m_LastOrder = m_spCQGCEL->CreateOrder(otMarket,
                                       m_Instrument,
                                       GetAccount(),
                                       abs(quantity),
                                       (quantity > 0 ? osdBuy : osdSell),
                                       0, 0, _bstr_t(CHelper::UE_NAME));

   m_LastOrder->Place();
}
